@@ -3,7 +3,7 @@
 //
 // has_iterator.h: Rcpp R/C++ interface class library -- identify if a class has a nested iterator typedef
 //
-// Copyright (C) 2010 - 2011 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2010 - 2012 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -28,13 +28,31 @@ class Argument ;
 
 namespace traits{
 
+template <typename T> struct needs_protection : false_type{} ;
+template <> struct needs_protection<SEXP> : true_type{} ;	
+
 template <typename T> class named_object {
 	public:
 		named_object( const std::string& name_, const T& o_) : 
-			name(name_), object(o_){} 
+		name(name_), object(o_){}
 		const std::string& name ;
 		const T& object ;
+		
 } ;
+template <> class named_object<SEXP> {
+public:
+	named_object( const std::string& name_, const SEXP& o_):
+	name(name_), object(o_){
+		R_PreserveObject(object) ;	
+	}
+	~named_object(){
+		R_ReleaseObject(object) ;	
+	}
+	const std::string& name ;
+	SEXP object ;
+} ;
+
+
 template <typename T> struct is_named : public false_type{} ;
 template <typename T> struct is_named< named_object<T> >   : public true_type {} ;
 template <> struct is_named< Rcpp::Argument >   : public true_type {} ;
