@@ -3,7 +3,7 @@
 // RcppCommon.h: Rcpp R/C++ interface class library -- common include and defines statements
 //
 // Copyright (C) 2008 - 2009 Dirk Eddelbuettel
-// Copyright (C) 2009 - 2012 Dirk Eddelbuettel and Romain Francois
+// Copyright (C) 2009 - 2013 Dirk Eddelbuettel and Romain Francois
 //
 // This file is part of Rcpp.
 //
@@ -23,11 +23,14 @@
 #ifndef RcppCommon_h
 #define RcppCommon_h
 
+// #define RCPP_DEBUG_LEVEL 1
+
 #include <Rcpp/platform/compiler.h>
 #include <Rcpp/config.h>
 #include <Rcpp/macros/macros.h>
 
 // include R headers, but set R_NO_REMAP and access everything via Rf_ prefixes
+#define MAXELTSIZE 8192
 #define R_NO_REMAP
 #include <R.h>
 #include <Rinternals.h>
@@ -36,6 +39,11 @@
 #include <R_ext/Rdynload.h>
 #include <Rversion.h>
 
+#ifdef __cplusplus
+extern "C" 
+#endif 
+const char * sexp_to_name(int sexp_type);
+
 /**
  * \brief Rcpp API
  */
